﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PictureMemoSystem001
{


    public delegate void PasteImageEventDelegate(Bitmap bmp, bool b_ignore, Color ignoreColor, Point pointToScreen, double opacity);

    public partial class FormCaptureAndPaste : Form
    {
        public event PasteImageEventDelegate PasteImageEvent;

        Color myTrans = Color.FromArgb(255, 254, 254, 254);
        

        bool b_debug = false;
        bool b_firsttime = true;

        public FormCaptureAndPaste()
        {
            InitializeComponent();

            initSub1();

            int maybeWidth = this.Width - this.ClientSize.Width;
            int maybeHeight = this.Height - this.ClientSize.Height;

            panel1.Width = 150;
            panel2.Height = 50;
            panel3.Width = 15;
            panel4.Height = 15;
            sizeAndPosition();
            this.MinimumSize = new Size(panel1.Width + panel3.Width + maybeWidth + 1, panel2.Height + panel4.Height + maybeHeight + 1);

            initSub2();
        }


        private void initSub1()
        {
            pictureBox1.BackColor = Color.Transparent;
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
            int wh = 10;
            Bitmap bmp = new Bitmap(wh, wh);
            for (int x = 0; x < wh; x++)
            {
                for (int y = 0; y < wh; y++)
                {
                    //bmp.SetPixel(x, y, myTrans);
                    bmp.SetPixel(x, y, Color.Transparent);
                }
            }
            //bmp.MakeTransparent(myTrans);
            pictureBox1.Image = bmp;

            Color temp = Color.FromKnownColor(KnownColor.Control);
            this.BackColor = myTrans;
            this.TransparencyKey = myTrans;
            panel1.BackColor = temp;
            panel2.BackColor = temp;
            panel3.BackColor = temp;
            panel4.BackColor = temp;
        }

        private void initSub2()
        {
            trackBarOpacity.Value = 100;
            buttonCaputure.Enabled = true;
            buttonPaste.Enabled = false;
            checkBoxIgnore.Enabled = false;
            numericUpDownA.Enabled = false;
            numericUpDownR.Enabled = false;
            numericUpDownG.Enabled = false;
            numericUpDownB.Enabled = false;
            checkBoxApplyOpacityWhenPaste.Enabled = false;
            buttonReset3.Visible = false;
        }

        private void sizeAndPosition()
        {
            if (b_firsttime)
            {
                panel1.Top = 0;
                panel1.Left = 0;

                panel2.Top = 0;
                panel2.Left = panel1.Right;

                panel3.Top = 0;
                panel4.Left = 0;
                b_firsttime = false;
            }

            
            panel3.Left = this.ClientSize.Width - panel3.Width;

            panel4.Top = this.ClientSize.Height - panel4.Height;
            

            panel1.Height = panel4.Top - panel1.Top;
            panel2.Width = panel3.Left - panel1.Right;
            panel3.Height = this.Height;
            panel4.Width = this.Width;

            pictureBox1.Top = panel2.Bottom;
            pictureBox1.Left = panel1.Right;

            pictureBox1.Width = panel3.Left - panel1.Right;
            pictureBox1.Height = panel4.Top - panel2.Bottom;
        }

        private void FormCaptureAndPaste_Resize(object sender, EventArgs e)
        {
            sizeAndPosition();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void panel1_DoubleClick(object sender, EventArgs e)
        {
            if (b_debug)
            {
                MessageBox.Show(pictureBox1.Size.ToString());
                MessageBox.Show(string.Format("{0},{1}", this.Bottom, panel4.Top));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCaputure_Click(object sender, EventArgs e)
        {
            if (pictureBox1.Width < 1 || pictureBox1.Height < 1)
            {
                MessageBox.Show("size error");
                return;
            }

            Bitmap bmp = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            Graphics g = Graphics.FromImage(bmp);

            Point p = pictureBox1.PointToScreen(new Point(0, 0));

            g.CopyFromScreen(p, new Point(0, 0), bmp.Size);

            /*
            Clipboard.SetDataObject(bmp);
            */

            pictureBox1.SizeMode = PictureBoxSizeMode.Normal;
            pictureBox1.Image = bmp;


            //SaveFileDialog sfd = new SaveFileDialog();
            //sfd.Filter = "ファイル(*.bmp)|*.bmp";
            //sfd.Title = "保存先のファイルを選択してください";

            //if (sfd.ShowDialog() == DialogResult.OK)
            //{
            //    bmp.Save(sfd.FileName, System.Drawing.Imaging.ImageFormat.Bmp);
            //    bmp.Dispose();
            //}
            g.Dispose();


            {
                buttonCaputure.Enabled = false;
                buttonPaste.Enabled = true;
                checkBoxIgnore.Enabled = true;
                numericUpDownA.Enabled = true;
                numericUpDownR.Enabled = true;
                numericUpDownG.Enabled = true;
                numericUpDownB.Enabled = true;
                checkBoxApplyOpacityWhenPaste.Enabled = true;
            }
            int x = 0;
            int y = 0;
            {
                numericUpDownA.Value = bmp.GetPixel(x, y).A;
                numericUpDownR.Value = bmp.GetPixel(x, y).R;
                numericUpDownG.Value = bmp.GetPixel(x, y).G;
                numericUpDownB.Value = bmp.GetPixel(x, y).B;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trackBarOpacity_ValueChanged(object sender, EventArgs e)
        {
            this.Opacity = (double)trackBarOpacity.Value / 100.0;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            int x = e.X;
            int y = e.Y;
            Bitmap bmp = (Bitmap)pictureBox1.Image;
            {
                try
                {
                    numericUpDownA.Value = bmp.GetPixel(x, y).A;
                    numericUpDownR.Value = bmp.GetPixel(x, y).R;
                    numericUpDownG.Value = bmp.GetPixel(x, y).G;
                    numericUpDownB.Value = bmp.GetPixel(x, y).B;
                }
                catch (Exception exc)
                {
                    // MessageBox.Show(exc.ToString());
                }
            }
        }

        private void buttonPaste_Click(object sender, EventArgs e)
        {
            Bitmap bmp = (Bitmap)pictureBox1.Image;
            bool b_ignore = checkBoxIgnore.Checked;
            Color ignorrColor = Color.FromArgb(
                (int)numericUpDownA.Value, 
                (int)numericUpDownR.Value, 
                (int)numericUpDownG.Value, 
                (int)numericUpDownB.Value);
            Point p = pictureBox1.PointToScreen(new Point(0, 0));

            double opacity = 1.0;
            if (checkBoxApplyOpacityWhenPaste.Checked)
            {
                opacity = (double)trackBarOpacity.Value / 100.0;
            }
            invokePasteImageEvent(bmp, b_ignore, ignorrColor, p, opacity);
            buttonReset3.Visible = true;
        }

        private void invokePasteImageEvent(Bitmap bmp, bool b_ignore, Color ignorrColor, Point p, double opacitt)
        {
            PasteImageEventDelegate temp = PasteImageEvent;

            if (temp != null)
            {
                temp(bmp, b_ignore, ignorrColor, p, opacitt);
            }
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            reset();
        }

        private void reset()
        {
            initSub1();
            initSub2();
            buttonReset3.Visible = false;
        }

        private void buttonReset2_Click(object sender, EventArgs e)
        {
            reset();
        }

        private void buttonReset3_Click(object sender, EventArgs e)
        {
            reset();
        }

    }
}
