﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace PictureMemoSystem001
{
    public delegate void FormSearchFindEventDelegate(string[] fullPahts);

    struct PathDtPair
    {
        public string path;
        public DateTime dateTime;

        public PathDtPair(string p1, DateTime p2)
        {
            path = p1;
            dateTime = p2;
        }
    }

    public partial class FormSearch : Form
    {
        string rootF;
        string[] Extensions;
        FormPMS Fpms;
        public event FormSearchFindEventDelegate FormSearchFindEvent;

        public FormSearch(string rootFolder,string[] extensions, FormPMS fpms)
        {
            InitializeComponent();
            rootF = rootFolder;
            Extensions = extensions;
            Fpms = fpms;
            monthCalendar1.MaxSelectionCount = 1;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonFindAndSetListbox_Click(object sender, EventArgs e)
        {
            this.rootF = Fpms.getRootFolderPath();
            List<string> pictures = getPicturesFullpath(rootF);
            pictures.Sort();

            List<PathDtPair> pdps = new List<PathDtPair>();                 
            foreach (string s in pictures)       
            {    
                pdps.Add(new PathDtPair(s, File.GetLastWriteTime(s)));      
            }

            int[] days;
            try
            {
                days = getDays();
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.ToString());
                return;
            }
            DateTime dt = monthCalendar1.SelectionStart;
            List<string> fullPaths = new List<string>();
            for (int i = 0; i < days.Length; i++)
            {
                fullPaths.Add(string.Format(" {0}",days[i]));
                DateTime dt2 = dt.AddDays((-1) * days[i]);
                foreach (PathDtPair pdp in pdps)
                {
                    if (dt2.Year == pdp.dateTime.Year &&
                        dt2.Month == pdp.dateTime.Month &&
                        dt2.Date == pdp.dateTime.Date)
                    {
                        fullPaths.Add(pdp.path);
                    }
                }
            }
            invokeFormSearchFineEvent(fullPaths.ToArray());
 
        }

        private void invokeFormSearchFineEvent(string[] fullPaths)
        {
            FormSearchFindEventDelegate temp = this.FormSearchFindEvent;

            if (temp != null)
            {
                temp(fullPaths);
            }
        }

        private List<string> getPicturesFullpath(string folderPath)
        {
            List<string> pictures = new List<string>();
            string[] files = Directory.GetFiles(folderPath);
            foreach (string file in files)
            {
                string ext = Path.GetExtension(file);
                for (int i = 0; i < Extensions.Length; i++)
                {
                    if (ext == Extensions[i])
                    {
                        pictures.Add(file);
                    }
                }
            }
            return pictures;
        }

        private int[] getDays()
        {
            int[] days;
            char[] separetor = {','};
            string[] sDays = textBox1.Text.Split(separetor);
            List<int> intsL = new List<int>();
            foreach(string s in sDays)
            {
                try
                {
                    intsL.Add(int.Parse(s));
                }
                catch(Exception e)
                {
                    throw e;
                }
            }
            days = intsL.ToArray();
            return days;
        }
    }
}
