﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace PictureMemoSystem001
{
    
    class LayerC
    {

        Color[,] dots;
        int dots_width;
        int dots_height;
        Color transparent;

        Color[,] aa_dots;

        // 描画命令の度に、どれを使うか教えてもらう。FillRectangleをのぞく。STAMPのぞく。
        public enum DrawLinePenKind { FIX_SQUARE_FILL, UNFIX_RECTANGLE_FILL, CIRCLE_FILL,
            FUNNY_BUG_UNFIX_RECTANGLE_FILL, 
            FUNNY_BUG2_UNFIX_RECTANGLE_FILL, FUNNY_BUG2B_UNFIX_RECTANGLE_FILL,
            CIRCLE_RANDOM_DOT_CONC,
            CIRCLE_RANDOM_DOT_MID,
            CIRCLE_RANDOM_DOT_SPARSE,
            USER_PEN_KIND_METHOD1,
            USER_PEN_KIND_METHOD2
        };

        bool[,] dotIsWritten;

        public LayerC(Size size, Color transparent)
        {
            this.dots = new Color[size.Width, size.Height];
            this.dots_width = size.Width;
            this.dots_height = size.Height;
            this.transparent = transparent;
            for (int x = 0; x < dots_width; x++)
            {
                for (int y = 0; y < dots_height; y++)
                {
                    this.dots[x, y] = this.transparent;
                }
            }
            this.aa_dots = new Color[dots_width, dots_height];
            this.dotIsWritten = new bool[dots_width, dots_height];
        }

        public LayerC(Bitmap bmp, Color transparent)
        {
            this.dots = new Color[bmp.Width, bmp.Height];
            this.dots_width = bmp.Width;
            this.dots_height = bmp.Height;
            this.transparent = transparent;
            for (int x = 0; x < dots_width; x++)
            {
                for (int y = 0; y < dots_height; y++)
                {
                    this.dots[x, y] = bmp.GetPixel(x, y);
                }
            }
            this.aa_dots = new Color[dots_width, dots_height];
            this.dotIsWritten = new bool[dots_width, dots_height];
        }

        public Bitmap getBitmap()
        {
            Bitmap bmp = new Bitmap(dots_width, dots_height);
            for (int x = 0; x < dots_width; x++)
            {
                for (int y = 0; y < dots_height; y++)
                {
                    bmp.SetPixel(x, y, dots[x, y]);
                }
            }
            return bmp;
        }

        // Layerに対する描画はこれがよばれる。
        public Rectangle DrawLine(Color c, Point startP, Point endP, int width, bool IsPENMODE_LINE, DrawLinePenKind dlpk, bool antialiasing)
        {
            Rectangle rect;
            switch (dlpk)
            {
                case DrawLinePenKind.CIRCLE_FILL:
                    rect = DrawLine_CIRCLE_FILL(c, startP, endP, width, IsPENMODE_LINE);
                    break;
                default:
                    rect = new Rectangle(0, 0, 0, 0);
                    break;
            }
            if (antialiasing)
            {
                AntiAliasing(rect);
            }
            return rect;
        }

        // 直径widthの円の中心をstartPに置き、endPまで中心を動かしていくような描画。
        private Rectangle DrawLine_CIRCLE_FILL(Color c, Point startP, Point endP, int width, bool IsPENMODE_LINE)
        {
            Rectangle rect = DrawLine_UNFIX_RECTANGLE_FILL_AND_CIRCLE_FILL(c, startP, endP, width, IsPENMODE_LINE, DrawLinePenKind.CIRCLE_FILL);
            return rect;

        }

        // 辺width * (width / ?)の長方形のwidthの辺の中心をstartPに置き、その辺がendPにぶつかるまで動かしていくような描画。
        // 長方形の辺はstartPとendPを結んだ線に平行または直行。
        //        
        // 直径widthの円の中心をstartPに置き、endPまで中心を動かしていくような描画。
        //
        private Rectangle DrawLine_UNFIX_RECTANGLE_FILL_AND_CIRCLE_FILL(Color c, Point startP, Point endP, int width, bool IsPENMODE_LINE, DrawLinePenKind dlpk)
        {

            double dx = (double)(endP.X - startP.X);
            double dy = (double)(endP.Y - startP.Y);
            double d = Math.Sqrt(dx * dx + dy * dy);

            if (dx == 0 && dy == 0)
            {
                Rectangle rect = new Rectangle(0, 0, 0, 0);
                return rect;
            }

            PointF[] startmagicpoints = getMagicPoints(startP, startP, endP, width);
            PointF endPF = new PointF(endP.X, endP.Y);

            if (dlpk == DrawLinePenKind.UNFIX_RECTANGLE_FILL)
            {
                //endPF.X += (float)((double)width * dx / d / 5.0);
                //endPF.Y += (float)((double)width * dy / d / 5.0);
            }

            PointF[] endmagicpoints = getMagicPoints(endPF, startP, endP, width);

            int smallX = (int)Math.Min(startmagicpoints[0].X, Math.Min(startmagicpoints[1].X, Math.Min(endmagicpoints[0].X, endmagicpoints[1].X))) - width / 2;
            int largeX = (int)Math.Max(startmagicpoints[0].X, Math.Max(startmagicpoints[1].X, Math.Max(endmagicpoints[0].X, endmagicpoints[1].X))) + width / 2;
            int smallY = (int)Math.Min(startmagicpoints[0].Y, Math.Min(startmagicpoints[1].Y, Math.Min(endmagicpoints[0].Y, endmagicpoints[1].Y))) - width / 2;
            int largeY = (int)Math.Max(startmagicpoints[0].Y, Math.Max(startmagicpoints[1].Y, Math.Max(endmagicpoints[0].Y, endmagicpoints[1].Y))) + width / 2;

            // 使いそうなところをfalseに
            for (int x = smallX; x <= largeX; x++)
            {
                for (int y = smallY; y <= largeY; y++)
                {
                    setDotIsWrittenFalse(x, y);
                }
            }

            for (int x = smallX; x <= largeX; x++)
            {
                for (int y = smallY; y <= largeY; y++)
                {
                    if (DotIsWritten(x, y))
                        continue;

                    bool include = false;
                    switch (dlpk)
                    {
                        case DrawLinePenKind.UNFIX_RECTANGLE_FILL:
                            include = isInclude(x, y, startmagicpoints, endmagicpoints);
                            break;
                        case DrawLinePenKind.CIRCLE_FILL:
                            include = isInclude(x, y, startmagicpoints, endmagicpoints) ||
                                 (x - startmagicpoints[0].X) * (x - startmagicpoints[0].X) + (y - startmagicpoints[0].Y) * (y - startmagicpoints[0].Y) + (x - startmagicpoints[1].X) * (x - startmagicpoints[1].X) + (y - startmagicpoints[1].Y) * (y - startmagicpoints[1].Y) <= width * width ||
                                 (x - endmagicpoints[0].X) * (x - endmagicpoints[0].X) + (y - endmagicpoints[0].Y) * (y - endmagicpoints[0].Y) + (x - endmagicpoints[1].X) * (x - endmagicpoints[1].X) + (y - endmagicpoints[1].Y) * (y - endmagicpoints[1].Y) <= width * width;
                            break;
                        default: break;
                    }


                    if (include)
                    {
                        dotOverWrite(x, y, c); 
                        setDotIsWrittenTrue(x, y);
                    }
                }
            }

            Rectangle rect2 = new Rectangle(smallX, smallY, largeX - smallX + 1, largeY - smallY + 1);
            return rect2;
        }

        private void AntiAliasing(Rectangle rect)
        {
            AntiAliasing1(rect);
        }

        /////////////////////////////////////////////////////////////////////////

        private void AntiAliasing1(Rectangle rect)
        {

            for (int x = rect.Left; x < rect.Right; x++)
            {
                if (x < 0) continue;
                if (x > this.dots_width - 1) continue;
                for (int y = rect.Top; y < rect.Bottom; y++)
                {
                    if (y < 0) continue;
                    if (y > this.dots_height - 1) continue;

                    int px = x;
                    int py = y;
                    int plx = x - 1; if (plx < 0) plx = 0;
                    int ply = y;
                    int prx = x + 1; if (prx > this.dots_width - 1) prx = this.dots_width - 1;
                    int pry = y;
                    int pux = x;
                    int puy = y - 1; if (puy < 0) puy = 0;
                    int pdx = x;
                    int pdy = y + 1; if (pdy > this.dots_height - 1) pdy = this.dots_height - 1;

                    bool need = IsNeedAntialiasing1(px, py, plx, ply, prx, pry, pux, puy, pdx, pdy);
                    CalcAntiAliasing1(px, py, plx, ply, prx, pry, pux, puy, pdx, pdy, need);
                }
            }

            for (int x = rect.Left; x < rect.Right; x++)
            {
                if (x < 0) continue;
                if (x > this.dots_width - 1) continue;
                for (int y = rect.Top; y < rect.Bottom; y++)
                {
                    if (y < 0) continue;
                    if (y > this.dots_height - 1) continue;
                    SetAntiAliasing(x, y);
                }
            }
        }

        private bool IsNeedAntialiasing1(int px, int py, int plx, int ply, int prx, int pry,
    int pux, int puy, int pdx, int pdy)
        {
            bool result = true;
            if (!IsMuchDifference1(px, py, plx, ply) && !IsMuchDifference1(px, py, prx, pry))
            {
                result = false;
            }
            if (!IsMuchDifference1(px, py, pux, puy) && !IsMuchDifference1(px, py, pdx, pdy))
            {
                result = false;
            }
            return result;
        }

        private bool IsMuchDifference1(int px, int py, int x, int y)
        {
            int much_diff = 5;
            bool result = false;
            if (Math.Abs(dots[px, py].A - dots[x, y].A) > much_diff ||
                Math.Abs(dots[px, py].R - dots[x, y].R) > much_diff ||
                Math.Abs(dots[px, py].G - dots[x, y].G) > much_diff ||
                Math.Abs(dots[px, py].B - dots[x, y].B) > much_diff)
            {
                result = true;
            }
            return result;
        }


        private void CalcAntiAliasing1(int px, int py, int plx, int ply, int prx, int pry,
            int pux, int puy, int pdx, int pdy, bool need)
        {
            if (need)
            {
                int weight = 5;
                int alpha = (this.dots[px, py].A * weight + this.dots[plx, ply].A + this.dots[prx, pry].A + this.dots[pux, puy].A + this.dots[pdx, pdy].A) / (weight + 4);
                //this.aa_alpha[px, py] = this.alpha[px, py];
                int red = (this.dots[px, py].R * weight + this.dots[plx, ply].R + this.dots[prx, pry].R + this.dots[pux, puy].R + this.dots[pdx, pdy].R) / (weight + 4);
                int green = (this.dots[px, py].G * weight + this.dots[plx, ply].G + this.dots[prx, pry].G + this.dots[pux, puy].G + this.dots[pdx, pdy].G) / (weight + 4);
                int blue = (this.dots[px, py].B * weight + this.dots[plx, ply].B + this.dots[prx, pry].B + this.dots[pux, puy].B + this.dots[pdx, pdy].B) / (weight + 4);
                aa_dots[px, py] = Color.FromArgb(alpha, red, green, blue);
            }
            else
            {
                int alpha = this.dots[px, py].A;
                int red = this.dots[px, py].R;
                int green = this.dots[px, py].G;
                int blue = this.dots[px, py].B;
                aa_dots[px, py] = Color.FromArgb(alpha, red, green, blue);
            }
        }

        private void SetAntiAliasing(int px, int py)
        {
            this.dots[px, py] = this.aa_dots[px, py];
        }

        private PointF[] getMagicPoints(PointF p, PointF startP, PointF endP, int width)
        {
            PointF[] points = new PointF[2];

            double dx = (double)(endP.X - startP.X);
            double dy = (double)(endP.Y - startP.Y);
            double d = Math.Sqrt(dx * dx + dy * dy);

            if (startP.X == endP.X)
            {
                points[0].X = p.X - (float)width / 2;
                points[1].X = p.X + (float)width / 2;
                points[0].Y = p.Y;
                points[1].Y = p.Y;
            }
            else if (startP.Y == endP.Y)
            {
                points[0].X = p.X;
                points[1].X = p.X;
                points[0].Y = p.Y - (float)width / 2;
                points[1].Y = p.Y + (float)width / 2;
            }
            else
            {
                points[0].X = (float)(p.X + ((dy / d) * (double)width / 2.0));
                points[0].Y = (float)(p.Y - ((dx / d) * (double)width / 2.0));
                points[1].X = (float)(p.X - ((dy / d) * (double)width / 2.0));
                points[1].Y = (float)(p.Y + ((dx / d) * (double)width / 2.0));
            }
            return points;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        private void setDotIsWrittenFalse(int x, int y)
        {
            if (x < 0 || x > dots_width - 1 || y < 0 || y > dots_height - 1)
                return;

            dotIsWritten[x, y] = false;

        }

        private void setDotIsWrittenTrue(int x, int y)
        {
            if (x < 0 || x > dots_width - 1 || y < 0 || y > dots_height - 1)
                return;

            dotIsWritten[x, y] = true;

        }

        private bool DotIsWritten(int x, int y)
        {
            if (x < 0 || x > dots_width - 1 || y < 0 || y > dots_height - 1)
                return false;
            return dotIsWritten[x, y];
        }

        ////
        // 辺からの長さでの判定はやめる。外積２種類で。
        private bool isInclude(int x, int y, PointF[] startpoints, PointF[] endpoints)
        {
            bool result = false;


            //if (isInclude(x, y, startpoints[0], startpoints[1], endpoints[0]))
            if (isInclude(x, y, startpoints[0], startpoints[1], endpoints[0]) &&
                isInclude(x, y, endpoints[1], startpoints[1], endpoints[0]))
            {
                result = true;
            }

            return result;
        }

        // 長方形を想定。
        // 外積を用いる。
        private bool isInclude(int x, int y, PointF o, PointF p1, PointF p2)
        {
            bool result = false;

            PointF vec = new PointF(x - o.X, y - o.Y);
            PointF vecP1 = new PointF(p1.X - o.X, p1.Y - o.Y);
            PointF vecP2 = new PointF(p2.X - o.X, p2.Y - o.Y);
            double L0 = Math.Sqrt((double)vec.X * (double)vec.X + (double)vec.Y * (double)vec.Y);
            double L1 = Math.Sqrt((double)vecP1.X * (double)vecP1.X + (double)vecP1.Y * (double)vecP1.Y);
            double L2 = Math.Sqrt((double)vecP2.X * (double)vecP2.X + (double)vecP2.Y * (double)vecP2.Y);

            if (L1 == 0.0 && L2 == 0.0)
            {
                result = false;
            }
            else if (L1 == 0.0)
            {
                result = false;
            }
            else if (L2 == 0.0)
            {
                result = false;
            }
            else
            {
                //double l1 = Math.Abs((double)crosPs(vec, vecP1) / L1);
                //double l2 = Math.Abs((double)crosPs(vec, vecP2) / L2);
                //if (l1 <= L2 && l2 <= L1)
                //{
                if (crosPs(vecP1, vecP2) >= 0 && crosPs(vecP1, vec) >= 0 && crosPs(vec, vecP2) >= 0)
                {
                    result = true;
                }
                else if (crosPs(vecP1, vecP2) <= 0 && crosPs(vecP1, vec) <= 0 && crosPs(vec, vecP2) <= 0)
                {
                    result = true;
                }

                //}
            }

            return result;
        }

        private float crosPs(PointF p1, PointF p2)
        {
            return p1.X * p2.Y - p2.X * p1.Y;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="c"></param>
        private void dotOverWrite(int x, int y, Color c)
        {
            if (x < 0 || x > dots_width - 1 || y < 0 || y > dots_height - 1)
                return;

                SetDot(x, y, c);
        }

        public void SetDot(int x, int y, Color c)
        {
            if (x < 0) x = 0;
            if (x > dots_width - 1) x = dots_width - 1;
            if (y < 0) y = 0;
            if (y > dots_height - 1) y = dots_height - 1;

            this.dots[x, y] = c;
        }

        public Color getDot(int x, int y)
        {
            if (x < 0)
                throw new Exception();
            if (x > dots_width - 1)
                throw new Exception();
            if (y < 0)
                throw new Exception();
            if (y > dots_height - 1)
                throw new Exception();

            return dots[x, y];
        }
    }
}
